@@ -17,13 +17,14 @@ class AccessLevel(enum.Enum):
 
 class ScenarioToolInterface:
 
-    def __init__(self, api_url="https://staging-api.dance4water.org/api",
-                 results_url="https://staging-sql.dance4water.org/resultsdb/"):
+    def __init__(self, api_url="https://stable-api.dance4water.org/api",
+                 results_url="https://stable-sql.dance4water.org/resultsdb/"):
         self.api_url = api_url
         self.results_url = results_url
 
         self.authenticated = False
         self.token = None
+        self._cache = {}
 
     def login(self, username: str, password: str):
         """
@@ -114,6 +115,10 @@ class ScenarioToolInterface:
         :param scenario_1_id: scenario id
         :return: data base id needed for query
         """
+        cache_key = f"get_database_id{scenario_id}"
+        if cache_key in self._cache:
+            return self._cache[cache_key]
+
         r = self.get_simulations(scenario_id)
         if r.status_code != 200:
             raise Exception(f"Unable to obtain scenarios {r.status_code}, {r.json()}")
@@ -123,7 +128,9 @@ class ScenarioToolInterface:
         for s in sims["simulations"]:
             sim = json.loads(s)
             if sim["simulation_type"] == "PERFORMANCE_ASSESSMENT":
-                return sim["id"]
+                self._cache = sim["id"]
+                return self._cache[cache_key]
+
 
     def create_project(self):
         """
@@ -174,11 +181,7 @@ class ScenarioToolInterface:
         """
         r = self.get_assessment_models()
 
-        if not r.status_code == 200:
-            raise Exception(f"Could not get assessment model {r.status_code}, {r.json()}")
-
-        models = r.json()["assessment_models"]
-
+        models = r["assessment_models"]
         filtered_nodes = []
         for model in models:
             if model['name'] == model_name:
@@ -209,7 +212,43 @@ class ScenarioToolInterface:
         if not r.status_code == 200:
             raise Exception(f"Unable to set project data model {r.status_code}, {r.json()}")
 
-    def create_scenario(self, project, parent, name="initialised model"):
+
+    def share_project(self, project, username):
+        """
+        Share a project
+
+        :param project: project id
+        :param username: share user name
+        """
+
+        data = {"username": username}
+
+        r = self._post(self.api_url + "/projects/" + str(project) + "/share", data)
+
+        if r.status_code == 200:
+            return
+
+        raise Exception(f"Unable to share project {r.status_code}, {r.json()}")
+
+    def archive_project(self, project):
+        """
+        Share a project
+
+        :param project: project id
+        :param username: share user name
+        """
+
+        data = {}
+
+        r = self._post(self.api_url + "/projects/" + str(project) + "/archive", data)
+
+        if r.status_code == 200:
+            return
+
+        raise Exception(f"Unable to archive project {r.status_code}")
+
+
+    def create_scenario(self, project, parent, name="Baseline"):
         """
         Creates a new scenario. The provides the shell for the new scenarios. Scenario are derived from the base line
         or any other scenario in the project. To modify the environment workflow may be defined and executed.
@@ -269,12 +308,15 @@ class ScenarioToolInterface:
 
         raise Exception(f"Something went wrong when adding the nodes {r.status_code} {r.json()}")
 
-    def get_scenario_workflow_nodes(self):
-        return self._get(self.api_url + "/sm_node/")
+    def get_scenario_workflow_nodes(self, scenario_id=None):
+        if scenario_id is None:
+            return self._get(self.api_url + "/sm_node/")
+        return self._get(self.api_url + "/scenario/" + str(scenario_id) + "/nodes")
 
     def create_dash_tile_template(self, json):
         """
         Define tile template (see doc)
+
         :param json:
         :return: dash_tile_template_id
         :rtype: int
@@ -290,6 +332,7 @@ class ScenarioToolInterface:
     def edit_dash_tile_template(self, template_id, json=None, active=True):
         """
         Define tile template (see doc)
+
         :param json:
         :return: dash_tile_template_id
         :rtype: int
@@ -309,6 +352,7 @@ class ScenarioToolInterface:
     def create_query_template(self, json, access_level):
         """
         Define tile template (see doc)
+
         :param json:
         :param access_level
         :type geojson: str
@@ -346,6 +390,7 @@ class ScenarioToolInterface:
     def link_query_prototype_assessment_model(self, query_prototype_id, assessment_model_id):
         """
         Link query prototype with assessment model
+
         :param query_prototype_id:
         :param assessment_model_id:
         :type query_prototype_id: int
@@ -364,6 +409,7 @@ class ScenarioToolInterface:
     def link_dash_tile_query_prototype(self, dash_tile_template_id , query_prototype_id):
         """
         Link query prototype with assessment model
+
         :param dash_tile_template_id:
         :param query_prototype_id:
         :type dash_tile_template_id: int
@@ -402,6 +448,57 @@ class ScenarioToolInterface:
 
         raise Exception(f"Unable to upload file {r.status_code}, {r.json()}")
 
+    def create_datasource(self,  project_id, name, source_type="file", upload_id=None):
+        """
+        Upload a geojson file and return id
+
+        :param geojson: geojson file
+        :param project_id: project the node will be assigned to
+        :param name: added option to set name of geojson file default is set to casestudyarea
+        :type geojson: str
+        :type name: str
+        :type project_id: int
+        :return: geojson id
+        :rtype: int
+        """
+
+        r = self._post(self.api_url + "/data_source/",
+                       {"project_id": project_id,
+                        "source_type": source_type,
+                        "upload_id": upload_id,
+                        "name": name})
+
+        if r.status_code == 200:
+            return r.json()["id"]
+
+        raise Exception(f"Unable to create datasource {r.status_code}, {r.json()}")
+
+    def upload_file(self, project_id, file):
+        """
+        Upload a  file and return id
+
+        :param file: path to file
+        :param project_id: project the file will be assigned to
+        :type file: str
+        :type name: str
+        :type project_id: int
+        :rtype: int
+        """
+
+        if not self.authenticated or self.token is None:
+            raise Exception(f"User not authenticated, {url}")
+
+        headers = {"Authorization": "Bearer " + self.token}
+
+        files = {'file': open(file, 'rb')}
+        r = requests.post(self.api_url + f"/upload/{project_id}", headers=headers, files=files)
+
+        if r.status_code == 200:
+            return r.json()["id"]
+
+        raise Exception(f"Unable to upload file {r.status_code}, {r.json()}")
+
+
     def get_region(self, region_name: str) -> int:
         """
         Returns region currently supported is Melbourne
@@ -728,6 +825,7 @@ class ScenarioToolInterface:
 
     def download_geojson(self, scenario_id, layer_name):
         """
+        Download layer as geojson file
 
         :param scenario_id: scenario id
         :param layern_name:
@@ -758,7 +856,8 @@ class ScenarioToolInterface:
 
         for s in sims["simulations"]:
             database_id = json.loads(s)["id"]
-        for s in sims["simulation_intances"]:
+
+        for s in sims["simulation_instances"]:
             print(json.loads(s)["id"], json.loads(s)["progress"], json.loads(s)["heartbeat"], json.loads(s)["log"])
 
         return database_id
@@ -903,6 +1002,7 @@ class ScenarioToolInterface:
         """
         Download project databases. Databases will be downloaded into folder/project_id.zip
         For larger projects it is recommended to defined the scenario_id to be downloaded. Otherwise the download might fail
+
         :param project_id: project id
         :param folder: folder
         :param scenario_id: scenario_id
@@ -926,6 +1026,7 @@ class ScenarioToolInterface:
     def get_my_status(self):
         """
         Get user status
+
         :return: dict with project status
         """
 
@@ -938,6 +1039,7 @@ class ScenarioToolInterface:
     def get_default_parameter_dict(self, node_id: int) -> dict:
         """
         Returns default parameter dict for a node
+
         :param node_id:
         :type node_id: int
         :return: Parameter dict
@@ -964,7 +1066,8 @@ class ScenarioToolInterface:
     def show_assessment_models(self) -> None:
         """
         List available assessment models
-        :return: Nonw
+
+        :return: None
         """
         models = self.get_assessment_models()
         for m in models['assessment_models']:
